#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#define version_string "v0.5 (c) 2014 ths@thorin.de"
#define pi 3.1415927
#define h  -0.0145 /* -50 bogenminuten (official) */
//#define h  -0.1046 /* -6 grad civil) */

float width=49.3225; // Breitengrad von Schwalbach/Saar
float length=6.8203; // Lngengrad von Schwalbach/Saar

char syscmd[255];

int month[13];

void init_months(int year)
{
  int schaltjahr=0;

  month[1]  = 31;
  month[2]  = 28;
  month[3]  = 31;
  month[4]  = 30;
  month[5]  = 31;
  month[6]  = 30;
  month[7]  = 31;
  month[8]  = 31;
  month[9]  = 30;
  month[10] = 31;
  month[11] = 30;
  month[12] = 31;

  /* Schaltjahr ? */
  if ((year%4)==0)   schaltjahr=1;
  if ((year%100)==0) schaltjahr=0;
  if ((year%400)==0) schaltjahr=1;

  if (schaltjahr) month[2] = 29;
}

/* Sommerzeit nachsehen */
void get_date(int n, int *day, int *mon)
{
  int i=1, sum=0, d=0, m=0;
  int past = 0;

  for (i=1; i<13; i++) {
    sum += month[i];
    if (sum>=n) {
      m = i;
      past = sum - month[i];
      break;
    }
  }

  d = n - past;

  //printf("%02d.%02d. \n", d, m);
  *mon = m; 
  *day = d;
}


/* Deklination berechnen */
float declin(int n)
{
  float d = 0.4095*sin(0.016906*(n-80.086)); // ergebnis in RAD

  return (d);
}

/* Zeitgleichung */
float timeequation (int n)
{
  float te = -0.171*sin(0.0337 * n + 0.465) - 0.1299*sin(0.01787 * n - 0.168); 

  return (te); 
}

/* Zeitdifferenz = 12*acos((sin(h) - sin(B)*sin(Deklination)) / (cos(B)*cos(Deklination)))/pi; */
void sunset(int n, int dst, char *cmd)
{
  int h2=0, m2=0, z=0;
  int monat=0, tag=0;

  float d = declin(n);
  float b = width * pi / 180;
  float te = timeequation(n);

  float td = 12*acos((sin(h) - sin(b)*sin(d)) / (cos(b)*cos(d)))/pi;
  float ld = (15 - length)*4/60; // Abweichung wegen Lngengrad

#ifdef SUNRISE
  float dn = 12 - td + ld + (-1) * te;
#else
  float dn = 12 + td + ld + (-1) * te;
#endif

  if (dst) z+=1;

#ifdef SUNRISE
  dn-=0.10; //  6 min vor Sonnenaufgang...
#else
  dn+=0.33; // 20 min nach Sonnenuntergang...
#endif
  
  h2 = (int)dn;
  m2 = (dn-h2)*60;

  get_date(n, &tag, &monat);

  /* mm hh mday mon wday      command */
  printf("%02d %02d %02d %02d *\troot\t%s\n", m2, h2+z, tag, monat, cmd);
}


int checkargs(int c, char **v)
{
  int o=0, n=0;

#ifdef SUNRISE
  sprintf(syscmd, "/root/bin/taster2"); /* auf */
#else
  sprintf(syscmd, "/root/bin/taster1"); /* zu */
#endif

  while ((o = getopt(c, v, "hvd:c:")) != -1) {
    switch (o) {
      case 'h':
        printf("Usage: %s [-v] [-h] [-d day_of_year] [-c command]\n", v[0]);
        exit(0);

      case 'v':
        printf("sunset %s\n", version_string);
        exit(0);

      case 'd':
        n = atoi(optarg);
        break;

      case 'c':
        memset(syscmd,0,255);
        strncpy(syscmd, optarg, 255);
        syscmd[254]=0;
        break;

      default: /* '?' */
        printf("Usage: %s [-v] [-c command] \n", v[0]);
        exit(0);
    }
  }

  return (n);
}

int main(int argc, char **argv)
{
  int n=1;
  struct tm *tm;
  time_t t;

  t = time(NULL);
  tm = localtime(&t);

  if (tm) {
    init_months(tm->tm_year + 1900);

    n = checkargs(argc, argv);

    if (n>0) {
      sunset(n,0,syscmd);
    } else {
      sunset(tm->tm_yday+1, tm->tm_isdst, syscmd);
    }
  }

  return (0);
}
